//---------------------------------------------------------
//RG Basic Tools Collection
//Copyright © 2003 - 2011 Ronald D. Grindle
//---------------------------------------------------------

/*
 * Created on Mai 14, 2011
 */
package com.qanavigation.Basics;

/**
 * Simple & fast date formatter. Processes ISO formatted Date (YYYY-MM-DDThh:mm) to
 * locale Strings. Does not have all locales implemented. Default locale is "iso";
 * Implemented locales:
 * de DD.MM.YYYY hh:mm:ss
 * iso (default)
 * @author rdgm3
 *
 */
public class DateFormatter {
	
	interface IDateFormatter {
		
		public String formDate( String p_isoDate );
		
		public String formDateLn( String p_isoDate );
	}
	
	private IDateFormatter m_dFormer;

	/**
	 * Unsupported locale is treated as "iso";
	 * @param p_locale e.g. "en", "de", "fr"
	 */
	public DateFormatter( String p_locale ) {

		if( p_locale.compareTo( "de" ) == 0 ) {

			m_dFormer = new DateFormatterDE();
		}
		else {

			/*if( p_locale.compareTo( "us" ) == 0 ) {

				m_dFormer = new DateFormatterEN();
			}
			else {*/

				m_dFormer = new DateFormatterDef();
			//}
		}
	}
	
	/**
	 * 
	 * @param p_isoDate ISO formatted Date (YYYY-MM-DDThh:mm)
	 * @return
	 */
	public String formDate( String p_isoDate ) {
		
		return m_dFormer.formDate( p_isoDate );
	}
	
	// ------------------
	private class DateFormatterB {
		
		protected boolean m_hasSec;
		
		// ---------------
		protected String[] split( String p_isoDate ) {
			
			int l;
			
			l = p_isoDate.length();
			
			String[] spl = new String[6];
			 
			if( l < 16 ) {
				
				spl[0] = "0000";
				spl[1] = "00";
				spl[2] = "00";
				// hh:mm:ss
				spl[3] = "00";
				spl[4] = "00";
			}
			else {
				
				// YYYY-MM-DD
				spl[0] = p_isoDate.substring( 0, 4 );
				spl[1] = p_isoDate.substring( 5, 7 );
				spl[2] = p_isoDate.substring( 8, 10 );
				// hh:mm:ss
				spl[3] = p_isoDate.substring( 11, 13 );
				spl[4] = p_isoDate.substring( 14, 16 );

				if( l > 16 ) {

					m_hasSec = true;
					spl[5] = p_isoDate.substring( 17, 19 );
				}
				else {

					m_hasSec = false;
					spl[5] = "";
				}
			}

			return spl;
		}
	}
	
	// ------------------	
	private class DateFormatterDE extends DateFormatterB implements IDateFormatter  {
		
		public String formDate( String p_isoDate ) {
			
			String res;
			String[] spl;
			
			spl = split( p_isoDate );
			
			res = spl[2] + "." + spl[1] + "." + spl[0] + " " + spl[3] + ":" + spl[4];
			
			return res;
		}
		
		public String formDateLn( String p_isoDate ) {
			
			String res;
			String[] spl;
			
			spl = split( p_isoDate );
			
			res = spl[2] + "." + spl[1] + "." + spl[0] + " " + spl[3] + ":" + spl[4] + ":" + spl[5]; 
			
			return res;
		}
	}
	
	// ------------------
	private class DateFormatterDef implements IDateFormatter {
		
		// -----------------
		public String formDate( String p_isoDate ) {
			
			String res;
			
			res = p_isoDate.replace( 'T', ' ' );
			
			res = res.substring( 0, 16 );
			
			return res;
		}
		
		// -----------------
		public String formDateLn( String p_isoDate ) {
			
			String res;
			
			res = p_isoDate.replace( 'T', ' ' );
			
			if( res.length() < 17 ) {
				
				res += ":00";
			}
			
			return res;
		}
	}

}
