//---------------------------------------------------------
// RG Basic Tools Collection
// Copyright © 2003,2004 Ronald D. Grindle
//---------------------------------------------------------
package com.qanavigation.XMLBasics;

import java.io.File;
import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.w3c.dom.Comment;
import org.w3c.dom.DOMException;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentType;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;

import com.qanavigation.Basics.Rslt;
import com.qanavigation.Basics.RsltStatVal;
import com.qanavigation.Basics.RsltString;
import com.qanavigation.Basics.TransRslt;


/***
 * XML with a result status.
 * RsltXMLDoc isA Rslt.
 */
public class RsltXMLDoc extends XMLRslt
{
	
	private DocumentBuilder m_DocBld;
	
	/**
	 * The XML document itself.
	 */
	public Document m_xDoc;

	//---------------------------------------------------------
	public RsltXMLDoc() {

		init();
	}
	
	//---------------------------------------------------------
	public RsltXMLDoc( String p_FileName ) {

		init();
		
		try {
		
			m_xDoc = m_DocBld.parse( new File( p_FileName ) );

			m_ResStat = RsltStatVal.isOK;
		}
		catch( SAXException excp ) {

			setRsltByExcp( excp );
		}
		catch( IOException excp ) {
						
			setRsltByExcp( excp );
		}
	}
	
	/**
	 * 
	 * @param p_URI The URI of the XML (optionally blank) 
	 * @param p_qName The top node name of the XML
	 */
	public RsltXMLDoc( String p_URI, String p_qName  ) {

		init();
		
		try {
						
			DOMImplementation DOMimpl = m_DocBld.getDOMImplementation();									
					
			m_xDoc = DOMimpl.createDocument( p_URI, p_qName, null );

			m_ResStat = RsltStatVal.isOK;
		}
		catch( DOMException excp ) {

			setRsltByExcp( excp );
		}
	}
	
	/**
	 * @param p_URI The URI of the XML (optionally blank ("")) 
	 * @param p_qName The top node name of the XML
	 * @param p_SystemDTD (The name of the DTD file) ('null' if entry is not to occur!)
	 * @param p_PublicDTD The public ID of the Docype ('null' if entry is not to occur!) 
	 */
	public RsltXMLDoc( String p_URI, String p_qName, String p_SystemDTD, String p_PublicDTD  ) {

		init();
		
		try {
						
			/*
			DocumentType svgDOCTYPE = impl.createDocumentType(
			  "svg", "-//W3C//DTD SVG 1.0//EN", 
			  "http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd"
			 );
			 Document doc = impl.createDocument(
			  "http://www.w3.org/2000/svg", "svg", svgDOCTYPE);*/
			
			DOMImplementation DOMimpl = m_DocBld.getDOMImplementation();
											
			DocumentType dType =  DOMimpl.createDocumentType( p_qName, p_PublicDTD, p_SystemDTD );
						
			m_xDoc = DOMimpl.createDocument( p_URI, p_qName, dType);

			m_ResStat = RsltStatVal.isOK;
		}
		catch( DOMException excp ) {

			setRsltByExcp( excp );
		}
	}
	
	/**
	 * Adds a comment to the xml
	 */
	public void addComment( String p_Comment ) {

		try {

			Comment xCom;
			
			xCom = m_xDoc.createComment( p_Comment );

			Element root = m_xDoc.getDocumentElement();
			
			m_xDoc.insertBefore( xCom, root );
		}
		catch( Exception excp ) {

			setRsltByExcp( excp );
			addRsltAddInfo( "'com.qanavigation.XMLBasics.RsltXMLDoc.addComment(..)' failed." );
		}
	}

	/** Retrieves a single value from the document.
	 * @param p_ValXPth The node path, separated by '/'
	 */
	public RsltString getNodeVal( String p_ValXPth ) {

		int s;
		
		RsltString rVal = new RsltString();
		
		String name;
	
		s = p_ValXPth.indexOf( "/" );
		
		RsltXMLNode xNode = new RsltXMLNode( m_xDoc.getDocumentElement() );		
					
		if( s == -1 ) {
				
			name = xNode.m_xNode.getNodeName();
			
			if( 0 == p_ValXPth.compareTo(name) ) {
			
				rVal = new RsltString( xNode.m_xNode.getNodeValue() );
			}
			else {
				
				rVal.setRsltStat( RsltStatVal.isEmpty );
			}
		}
		else {
				
			String path;
			
			name = p_ValXPth.substring( 0, s );
			
			path = p_ValXPth.substring( s + 1 );
			
			rVal = xNode.getSubNodeVal( path );	
		}
		
		return rVal;
	}
	
	/**
	 * 
	 * @param p_StyleSrc The XSLT source.
	 * @param p_Out Target stream.
	 * @return
	 */
	public Rslt transform( StreamSource p_StyleSrc, StreamResult p_Out ) {		
		
		TransRslt res;
		
		res = new TransRslt( true );
		//TransfErrListener errLst; 
		
		//errLst = new TransfErrListener( res );
		
		try {
			
 			//File stylesheet = new File( p_StyleFileName );
  		   
			TransformerFactory tFactory = TransformerFactory.newInstance();

			Transformer transformer = tFactory.newTransformer(p_StyleSrc);
   		
			//transformer.setErrorListener( res );
   		
			//ErrorListener errLstn = transformer.getErrorListener();
			/*
			transformer.setOutputProperty(OutputKeys.MEDIA_TYPE, "text/html");
			transformer.setOutputProperty(OutputKeys.ENCODING, "ISO-8859-1");
			transformer.setOutputProperty(OutputKeys.METHOD, "html");
			*/	
				   	   	
			DOMSource source = new DOMSource( m_xDoc );
			
			transformer.transform( source, p_Out );      
		}
//		TransformerException is worth a look to retrieve more information
		catch( TransformerException texcp ) {
			
			res.setRsltByExcp( texcp );
			
			res.addRsltAddInfo( "RsltXMLDoc.transform() failed." ); 
		}
		
		return res;
	}
	
	/**
	 * Gets the top node of the document.
	 * @return The top node.
	 */	
	public RsltXMLNode getTopNode( ) {
		
		RsltXMLNode rNode = new RsltXMLNode( m_xDoc.getDocumentElement() );		
		
		return rNode;
	}
	
	// ---------------------------------------------------------
	protected void init() {
		
		super.init();
		
		DocumentBuilderFactory dBF = DocumentBuilderFactory.newInstance();
		
		dBF.setValidating( false );

		dBF.setIgnoringComments( true );

		dBF.setIgnoringElementContentWhitespace( true );
		
		m_xDoc = null;
		m_DocBld = null;
		
		try {

			m_DocBld = dBF.newDocumentBuilder();
		}
		catch ( ParserConfigurationException pce ) {

	//System.err.println( pce );
				setRsltByExcp( pce );
				setInError( "Creation of DocumentBuilder failed." );
		}

		//m_DBld.setErrorHandler( new XErrorHandler( this ) );

		m_ResStat = RsltStatVal.isInit;
	}
	
	/**
	 * Safe the XML to the given file.
	 * @param p_FileName
	 */
	public void save( String p_FileName ) {
		
		try {
			
			File file = new File( p_FileName );

			// Use a Transformer for output
		  TransformerFactory tFactory = TransformerFactory.newInstance();
		  Transformer transformer = tFactory.newTransformer();

		  DOMSource source = new DOMSource( m_xDoc );
		   
		  //StreamResult result = new StreamResult(System.out);
		  StreamResult result = new StreamResult( file );
		   
		  DocumentType dType = m_xDoc.getDoctype(); 
		   
		  if ( dType != null ) {
		   	
//				String systemValue = (new File(document.getDoctype().getSystemId())).getName();

		  	String val = dType.getSystemId();
				
				if( null != val ) {

					if( val.length() > 0 ) {
						
						transformer.setOutputProperty( OutputKeys.DOCTYPE_SYSTEM, val );					
					}
				}
				
				val = dType.getPublicId();
				
				if( null != val ) {
					
					if( val.length() > 0 ) {
							
						transformer.setOutputProperty( OutputKeys.DOCTYPE_PUBLIC, val );					
					}
				}
		  }				
					   
		  transformer.transform( source, result );
		} 
		catch (TransformerConfigurationException tce) {

			addRsltAddInfo( "* Transformer Factory error" );
			setRsltByExcp( tce );
		} 
		catch (TransformerException te) {

			// Error generated by the parser
			addRsltAddInfo( "* Transformation error" );
			setRsltByExcp( te );
		}	
	}
}
