//---------------------------------------------------------
// RG XML DB Tools Collection
// Copyright © 2003,2004 Ronald D. Grindle
//---------------------------------------------------------

package com.qanavigation.XMLDB;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.ProcessingInstruction;

import com.qanavigation.Basics.RsltStatVal;
import com.qanavigation.Basics.RsltString;
import com.qanavigation.Basics.RsltTable;

import com.qanavigation.XMLBasics.RsltXMLDoc;
import com.qanavigation.XMLBasics.RsltXMLNode;


/**
 * XMLDBNode is the main object to deal with when working with XMLDB.
 * While it acts as a node it is at the same time a document.
 */
public class XMLDBNode extends RsltXMLNode
{
	
	public RsltXMLDoc m_RxDoc;
	
	public String m_UID;
	
	private XMLDBbasics m_xDB;

	/**
	 * 
	 * @param p_xDB
	 * @param p_UID
	 */
	public XMLDBNode( XMLDBbasics p_xDB, String p_UID )
	{
		
		RsltTable nodeTbl;
		String nodeName;
		String[]	dtdInfo;
		
		m_xDB = p_xDB;
		m_UID = p_UID;

		nodeTbl = m_xDB.getNodeAsTable( p_UID );
		
		if( nodeTbl.notOKorEmpty() ) {
			
			addRsltAddInfo( "XMLDBNode(): Node not found." );

			m_RxDoc = new RsltXMLDoc( "Dummy",  "Dummy", "Dummy", "Dummy");
					
			super.init( m_RxDoc.getTopNode() ); //Sets it OK!
			
			setRsltStat( nodeTbl );
		}
		else {

			nodeName = nodeTbl.getCell( 1, 0 );
			
			dtdInfo = m_xDB.getDTDInfo( p_UID );

			m_RxDoc = new RsltXMLDoc( null,  nodeName, dtdInfo[0], dtdInfo[1] );
			
			/* 2004-06-06
			RsltXMLNode me = this;
			appAttrs( me, p_UID );
			*/
			
			super.init( m_RxDoc.getTopNode() );
		}
		
		if( notOK() ) {

			addRsltAddInfo( "'RG.XMLDB.XMLDBNode.XMLDBNode(..)' failed." );
		}
	}
	
	/**
	 * @param p_Com
	 */
	public void addComment( String p_Com ) {
		
		m_RxDoc.addComment( p_Com ); 
	}

	/**
	 * Appends the attributes of a node.
	 * @param p_xNode
	 * @param p_UID
	 */
	public void appAttrs( RsltXMLNode p_xNode, String p_UID ) {

		int i, x;

		RsltTable attrTbl;

		try {

			p_xNode.addAttribute( XMLDBDefs.c_AttrNmUID, p_UID );

			attrTbl = m_xDB.getNodeAttrs( p_UID );

			if( attrTbl.isOK( true ) ) {

				x = attrTbl.getRowNum();
			
				for( i = 0; i < x; i++ ) {

					p_xNode.addAttribute( attrTbl.getCell( 0, i ), attrTbl.getCell( 1, i ));
				}
			}
		}
		catch( Exception excp ) {

			setRsltByExcp( excp );
		}
		
		if( notOK() ) {

			addRsltAddInfo( "'RG.XMLDB.XMLDBNode.appAttrs(..)' failed." );
		}
	}

	// --------------------------------------------
	private void appSubNodes( RsltXMLNode p_parentNode, String p_parentUID ) {

		int i, x;

		RsltString val;
		RsltTable nodeTbl;

		RsltXMLNode newNode;

		String nodeName;
		String strNodeType;
		String UID;

		try {

			appAttrs( p_parentNode, p_parentUID );

			if( isOK() ) {

				appVal( p_parentNode, p_parentUID );
			}

			if( isOK() ) {

				nodeTbl = m_xDB.getXNodeLstByParent( p_parentUID );

				if( nodeTbl.isOK() ) {

					x = nodeTbl.getRowNum();
				
					for( i = 0; i < x; i++ ) {

						UID = nodeTbl.getCell( 0, i );
						nodeName = nodeTbl.getCell( 1, i );				
						strNodeType = nodeTbl.getCell( 2, i );

						short nodeType;

						nodeType = Short.parseShort( strNodeType );

						switch( nodeType ) {

/* unknown in Java
							case Node.XmlDeclaration:

								val = m_xDB.getNodeVal( UID );

								if( val.notOK () ) {

									setResStat( val );
								}
								else {

									p_parentNode.createXmlDecl( "1.0", "", "" );
								}
								
							break; */
							

							//case XMLDBDefs.c_NodeTypeProcInstr:
							case Node.PROCESSING_INSTRUCTION_NODE:

								val = m_xDB.getNodeVal( UID );

								if( val.notOK () ) {

									setRsltStat( val );
								}
								else {

									Document doc;
									ProcessingInstruction procInstr;
									
									doc =  p_parentNode.m_xNode.getOwnerDocument();
																		
									procInstr = doc.createProcessingInstruction( nodeName, val.getString() );
									
									Element root = doc.getDocumentElement();
									
									doc.insertBefore( procInstr, root );
								}
								break;

							//case XMLDBDefs.c_NodeTypeDocType:
							case Node.DOCUMENT_TYPE_NODE:
/* must be detecetd and creaed before the creation of the document
 * is ignored
 
								val = m_xDB.getNodeVal( UID );

								if( val.notOK () ) {

									setResStat( val );
								}
								else {

									createDocType( p_parentNode, nodeName, val );
								}
*/								
							break;

							default:

								newNode = p_parentNode.appNode( nodeName );	
					
								if( newNode.notOK() ) {

									setRsltStat( newNode );
								}
								else {

									appSubNodes( newNode, UID );
								}
								break;
						}

						if( notOK() ) {

							i = x;
						}
					}
				}
			}
		}
		catch( Exception excp ) {

			setRsltByExcp( excp );
		}
		
		if( notOK() ) {

			addRsltAddInfo( "'RG.XMLDB.XMLDBNode.appSubNodes(..)' failed." );
		}
	}

	/**
	 * 
	 * @param p_xNode
	 * @param p_UID
	 */
	public void appVal( RsltXMLNode p_xNode, String p_UID ) {

		RsltString val;

		val = m_xDB.getNodeVal( p_UID );

		if( val.isOK() ) {

			p_xNode.appTxt( val.getString() );
		}
		else {

			if( RsltStatVal.isEmpty != val.getRsltStatVal() ) {

				setRsltStat( val );
			}
		}

		if( notOK() ) {

			addRsltAddInfo( "'RG.XMLDB.XMLDBNode.appVal(..)' failed." );
		}
	}

	/**
	 * Detects the subtree of the node and add them to the XML Doc.
	 */
	public void gatherSubTree( ) { 

		RsltXMLNode me = this;

		appSubNodes( me, m_UID );

		if( notOK() ) {

			addRsltAddInfo( "'RG.XMLDB.XMLDBNode.gatherSubTree(..)' failed." );
		}
	}
}
