/*MySQL version*/

CREATE TABLE Node(	UID CHAR(27) UNIQUE NOT NULL,
										UIDpNode CHAR(27),
										NodeType CHAR(32) NOT NULL,
										Prefix VARCHAR(255), 
										Name VARCHAR(255) NOT NULL,
										INDEX( UID ),
										INDEX( UIDpNode ),
										PRIMARY KEY (UID),
										FOREIGN KEY (UIDpNode) REFERENCES Node( UID ) /* ON DELETE CASCADE */
									) TYPE = innodb;

CREATE TABLE Attrib(	UIDpNode CHAR(27) NOT NULL,
											Prefix VARCHAR(255),
											Name VARCHAR(255) NOT NULL,
											Value BLOB,
											INDEX ( UIDpNode, Name ),
											PRIMARY KEY ( UIDpNode, Name ),
											FOREIGN KEY (UIDpNode) REFERENCES Node( UID ) ON DELETE CASCADE
										) TYPE = innodb;

CREATE TABLE NodeVal( UIDpNode CHAR(27) UNIQUE NOT NULL,
											Value BLOB,
											INDEX( UIDpNode ),
											FOREIGN KEY (UIDpNode) REFERENCES Node( UID ) ON DELETE CASCADE
										) TYPE = innodb;

CREATE TABLE XMLFile(	UIDpNode CHAR(27) UNIQUE NOT NULL,
											Path BLOB,
											Name VARCHAR(255) NOT NULL,
											INDEX( UIDpNode ),
											FOREIGN KEY (UIDpNode) REFERENCES Node( UID ) ON DELETE CASCADE
										) TYPE = innodb;

CREATE TABLE FileData(	UID CHAR(27) UNIQUE NOT NULL,
												UIDpNode CHAR(27),
												ContentType CHAR(32),
												DateTime CHAR(16), 
												Name VARCHAR(512) NOT NULL,
												Path VARCHAR(1024),
												Size CHAR(32),
												Content BLOB, 
												INDEX( UID ),
												INDEX( UIDpNode ),
												PRIMARY KEY (UID),
												FOREIGN KEY (UIDpNode) REFERENCES Node( UID ) /* ON DELETE CASCADE */
											) TYPE = innodb;

/* Dummy entry for topmost nodes*/
INSERT INTO Node( UID, UIDpNode, Name, NodeType ) VALUES ( '0', '0', 'DummyTopNode', '-1' );
